@@ -1,8 +1,8 @@
-from pyg_base import is_date, ulist, logger, is_str, as_list, get_cache, Dict, dictable
+from pyg_base import is_date, ulist, logger, is_str, as_list, get_cache, Dict, dictable, list_instances
 from pyg_encoders import cell_root, root_path, pd_read_parquet, pickle_load, pd_read_csv, dictable_decode
 from pyg_npy import pd_read_npy
 from pyg_cell._types import _get_mode, _get_qq, DBS, QQ
-from pyg_cell._cell import cell, cell_clear, cell_item, cell_output
+from pyg_cell._cell import cell, cell_clear, cell_item, cell_output, cell_func
 from pyg_cell._dag import get_DAG, get_GAD, descendants
 
 from functools import partial
@@ -49,6 +49,25 @@ def is_pairs(pairs):
     return isinstance(pairs, tuple) and min([isinstance(item, tuple) and len(item) == 2 for item in pairs], default= False)
 
 
+class db_cell_func(cell_func):
+    """
+    same as cell_func but implements a check that all cells internally point to the same environment variables
+
+    """
+    def wrapped(self, *args, **kwargs):
+        env = self.get('env')
+        if env is not None:
+            cells = list_instances((args, kwargs), db_cell)
+            for c in cells:
+                if 'db' in c.keys():
+                    keywords = c['db'].keywords
+                    for k,v in env.items():
+                        if keywords.get(k)!=v:
+                            raise ValueError('cell is pointing to %s=%s while all cells must point to %s'%(k,keywords.get(k),v))
+        return  super(db_cell_func, self).wrapped(*args, **kwargs)           
+
+
+        
 def db_save(value):
     """
     saves a db_cell from the database. Will iterates through lists and dicts
@@ -105,9 +124,9 @@ def _load_asof(table, kwargs, deleted, qq = None):
     l = len(live)
     if deleted in (False, None): # we just want live values
         if l == 0:
-            raise ValueError('no undeleted cells found matching %s'%kwargs)        
+            raise ValueError('no undeleted cells found matching %s in \n%s'%(kwargs, live))        
         elif l>1:
-            raise ValueError('multiple cells found matching %s'%kwargs)
+            raise ValueError('multiple cells found matching %s in \n%a'%(kwargs, live))
         return live[0]
     else:  
         qq = _get_qq(qq, table)
@@ -117,7 +136,7 @@ def _load_asof(table, kwargs, deleted, qq = None):
             return past[0]
         elif p > 1:
             if deleted is True:
-                raise ValueError('multiple historic cells are avaialble %s with these dates: %s. set delete = DATE to find the cell on that date'%(kwargs, past.deleted))
+                raise ValueError('multiple historic cells are avaialble %s with these dates: %s. set delete = DATE to find the cell on that date in \n%s'%(kwargs, past.deleted, past))
             else:
                 return past.sort('deleted')[0]
         else:    ## no records found in past, we go to the deleted history
@@ -125,12 +144,12 @@ def _load_asof(table, kwargs, deleted, qq = None):
             h = len(history)
             if h == 0:
                 if l > 0:
-                    raise ValueError('no deleted cells found matching %s but a live one exists. Set deleted = False to get it'%kwargs)        
+                    raise ValueError('no deleted cells found matching %s but a live one exists. Set deleted = False to get it, in \n%s'%(kwargs, history))        
                 else:                   
-                    raise ValueError('no deleted cells found matching %s'%kwargs)        
+                    raise ValueError('no deleted cells found matching %s in \n%s'%(kwargs, history))        
             elif h>1:
                 if deleted is True:
-                    raise ValueError('multiple historic cells are avaialble %s with these dates: %s. set delete = DATE to find the cell on that date'%(kwargs, history.deleted))
+                    raise ValueError('multiple historic cells are avaialble %s with these dates: %s. set delete = DATE to find the cell on that date, in \n%s'%(kwargs, history.deleted, history))
                 else:
                     return history.sort('deleted')[0]
             else:
@@ -143,7 +162,7 @@ def _is_primitive(value):
     elif isinstance(value, dict):
         return min([_is_primitive(v) for v in value.values()], default = True)
     else:
-        return value is None or isinstance(value, (bool, str, int, float, datetime.datetime, datetime.date, partial))
+        return value is None or isinstance(value, (type, bool, str, int, float, datetime.datetime, datetime.date, partial))
 
 class db_cell(cell):
     """
@@ -211,6 +230,25 @@ class db_cell(cell):
             self[_db] = None
             super(db_cell, self).__init__(function = function, output = output, **kwargs)
 
+    _func = db_cell_func
+    
+    @property
+    def _function(self):
+        if 'db' not in self:
+            function = self.function if isinstance(self.function, cell_func) else cell_func(self.function)
+        else:
+            keywords = self.db.keywords
+            env = {}
+            for k in ['server', 'url']:
+                if k in keywords:
+                    env[k] = keywords[k]
+            if env:
+                function = self.function if isinstance(self.function, db_cell_func) else db_cell_func(self.function)
+                function.env = env
+            else:
+                function = self.function if isinstance(self.function, cell_func) else cell_func(self.function)                
+        return function
+
     @property
     def _pk(self):
         if self.get(_db) is None:
@@ -256,7 +294,7 @@ class db_cell(cell):
         if self.get(_db) is None: 
             return super(db_cell, self)._clear()
         else:
-            return self[[_db, _function, _updated] + self.db().pk] if _updated in self else self[[_db, _function] + self.db().pk]
+            return self[[_db, _function, _updated] + self.db()._pk] if _updated in self else self[[_db, _function] + self.db()._pk]
 
     def save(self):
         if self.get(_db) is None:
@@ -479,8 +517,11 @@ def _get_cell(table = None, db = None, url = None, server = None, deleted = None
 
     :Parameters:
     ----------
-    table : str
-        name of table (Mongo collection). alternatively, you can just provide an address
+    table : str, an address in the form of pairs, or a partial to sql_table on mongo_table
+        name of table (Mongo collection). 
+        alternatively, you can just provide an address
+        or a partial to the actual table
+        
     db : str
         name of database.
     url : str, optional
@@ -527,18 +568,30 @@ def _get_cell(table = None, db = None, url = None, server = None, deleted = None
     
     schema = kwargs.pop('schema', None)
     mode = _get_mode(url, server, schema = schema)    
-    if table is not None and (db is not None or mode == 'sql'):
-        if mode == 'mongo':
-            t = DBS[mode](db = db, table = table, url = url or server, pk = pk)
-            qq = QQ[mode]
-        elif mode == 'sql':
-            t = DBS[mode](db = db, table = table, server = server or url, schema = schema, pk = pk, doc = doc or _doc)
-            qq = t.table.c
+    if table is not None:
+        if isinstance(table, partial):
+            t = table()
+            if table.func == DBS.get('sql'):
+                mode = 'sql'
+                qq = t.table.c
+            elif table.func == DBS.get('mongo'):
+                mode = 'mongo'
+                qq = QQ[mode]
+        elif (db is not None or mode == 'sql'):
+            if mode == 'mongo':
+                t = DBS[mode](db = db, table = table, url = url or server, pk = pk)
+                qq = QQ[mode]
+            elif mode == 'sql':
+                t = DBS[mode](db = db, table = table, server = server or url, schema = schema, pk = pk, doc = doc or _doc)
+                qq = t.table.c
+        else:
+            return GRAPH[address]
         address = t.address + kwargs_address
         if _from_memory and deleted in (None, False): # we want the standard cell
             if address not in GRAPH:
                 doc = _load_asof(t, kwargs, deleted, qq)
-                GRAPH[doc._address] = doc
+                if isinstance(doc, cell):
+                    GRAPH[doc._address] = doc
                 return doc
             else:
                 return GRAPH[address]
